import unittest2 as unittest

testmodules = (
    'tests.imports',
    'tests.settings',
    'tests.discovery',
    'tests.autoreload',
)

def runtests():
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    suite.addTests(loader.loadTestsFromNames(testmodules))
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest2 as unittest
from tests.base import BaseTestCase
import os

class DiscoveryTestCase(BaseTestCase):

    def setUp(self):
        super(DiscoveryTestCase, self).setUp()
        os.environ.setdefault(self.envvar,
            "%s.settings" % self.test_project)

    def test_project_path(self):
        """Test project path"""
        import flexisettings.settings
        self.assertEqual(
            flexisettings.settings.FLEXI_PROJECT_ROOT,
            os.path.join(self.current_dir, self.test_folder)
        )

    def test_sys_path(self):
        """Test layout discovery"""
        import flexisettings.settings
        import sys
        self.assertIn(
            os.path.join(flexisettings.settings.FLEXI_PROJECT_ROOT, 'apps'),
            sys.path
        )

    def test_discovery_media(self):
        """Test media folder discovery"""
        import flexisettings.settings
        self.assertEqual(
            flexisettings.settings.MEDIA_ROOT,
            os.path.join(self.current_dir, self.test_folder, 'media')
        )

    def test_discovery_static(self):
        """Test static folder discovery"""
        import flexisettings.settings
        self.assertEqual(
            flexisettings.settings.STATIC_ROOT,
            os.path.join(self.current_dir, self.test_folder, 'static')
        )

    def test_discovery_templates(self):
        """Test templates folder discovery"""
        import flexisettings.settings
        self.assertIn(
            os.path.join(self.current_dir, self.test_folder, 'templates'),
            flexisettings.settings.TEMPLATE_DIRS,
        )

if __name__ == '__main__':
    unittest.main()



import unittest2 as unittest
from tests.base import BaseTestCase
import os

class AutoreloadTestCase(BaseTestCase):

    def setUp(self):
        super(AutoreloadTestCase, self).setUp()
        os.environ.setdefault(self.envvar,
            "%s.settings" % self.test_project)

    def test_autoreload(self):
        """Test if settings modules are available through sys.modules"""
        import flexisettings.settings
        import sys
        self.assertIn(
            '.'.join(
                ['flexisettings.wrapped', self.test_project, 'settings_t']),
            sys.modules
        )

    def test_module_file(self):
        """Test settings modules __file__ value"""
        import flexisettings.settings
        import sys
        self.assertEqual(
            os.path.join(
                self.current_dir,
                self.test_folder,
                self.test_project,
                'settings_t.py'),
            sys.modules['.'.join(
                ['flexisettings.wrapped', self.test_project, 'settings_t'])].__file__
        )


if __name__ == '__main__':
    unittest.main()



import unittest2 as unittest
from tests.base import BaseTestCase
import os

class SettingsTestCase(BaseTestCase):

    def setUp(self):
        super(SettingsTestCase, self).setUp()
        os.environ.setdefault(self.envvar,
            "%s.settings" % self.test_project)

    def test_run_env(self):
        """Test running environment lookup"""
        import flexisettings.settings
        self.assertEqual(flexisettings.settings.FLEXI_RUN_ENV, 't')

    def test_debug_settings(self):
        """Test proxyfied lookup without evaluation (settings.DEBUG)"""
        import flexisettings.settings
        self.assertTrue(flexisettings.settings.DEBUG)

    def test_security_settings(self):
        """Test proxyfied lookup with evaluation (settings.SECRET_KEY)"""
        import flexisettings.settings
        self.assertEqual(flexisettings.settings.SECRET_KEY, self.secret_key)

if __name__ == '__main__':
    unittest.main()



import unittest2 as unittest
import sys
import os
import shutil

from django.core.management import call_command


class BaseTestCase(unittest.TestCase):

    test_folder = 't'
    test_project = 'testProject'
    envvar = 'FLEXI_WRAPPED_MODULE'

    secret_key = 'adummysecretkey'

    @classmethod
    def setUpClass(cls):
        # get current folder
        cls.current_dir = os.getcwd()
        if not os.path.isdir(cls.test_folder):
            # create test folder
            os.mkdir(cls.test_folder)
            # create a sample project
            call_command('startproject', cls.test_project, cls.test_folder)
            # create environment configuration
            env_fd = open(
                os.path.join(cls.test_folder, cls.test_project, 'env.py'),
                'w')
            env_fd.write("FLEXI_RUN_ENV = 't'\n")
            env_fd.close()
            # create security file
            sec_fd = open(
                os.path.join(cls.test_folder, cls.test_project, 'security.py'),
                'w')
            sec_fd.write("_SECRET_KEY = '%s'\n" % cls.secret_key)
            sec_fd.close()
            # create local settings file
            locset_fd = open(
                os.path.join(cls.test_folder, cls.test_project, 'settings_t.py'),
                'w')
            locset_fd.write("SECRET_KEY = _SECRET_KEY\n")
            locset_fd.write("FLEXI_LAYOUT_DISCOVERY = True\n")
            locset_fd.close()
            # add local apps, media, static, templates folders
            os.mkdir(os.path.join(cls.test_folder, 'apps'))
            os.mkdir(os.path.join(cls.test_folder, 'media'))
            os.mkdir(os.path.join(cls.test_folder, 'static'))
            os.mkdir(os.path.join(cls.test_folder, 'templates'))
            # add this location to sys.path for import
            sys.path.insert(0, os.path.join(os.getcwd(), cls.test_folder))

    def setUp(self):
        # change current directory to test folder
        os.chdir(self.test_folder)

    def tearDown(self):
        if self.envvar in os.environ:
            os.environ.pop(self.envvar)
        os.chdir('..')

    @classmethod
    def tearDownClass(cls):
        os.chdir(cls.current_dir)
        shutil.rmtree(cls.test_folder)
        # THIS IS AN UGLY HACK
        # if test_project is not removed, the next 'startproject'
        # command will fail. It is not possible at the present time to
        # really unload python modules.
        if cls.test_project in sys.modules:
            sys.modules.pop(cls.test_project)



# -*- coding: utf-8 -*-
__title__ = u'django-flexisettings'
# semantic versioning, check http://semver.org/
__version__ = u'0.2.5'
__author__ = u'Stefan Berder <stefan.berder@ledapei.com>'
__copyright__ = u'Copyright 2012-2013, Heatwave fashion Ltd., ' \
    u'all rights reserved'



import unittest2 as unittest
import sys
import os

from tests.base import BaseTestCase


class ImportTestCase(BaseTestCase):

    # it is necessary to run those tests in that order to avoid
    # namespace pollution with imported module
    tests = ['test_import_without_env', 'test_import', 'test_import_local']

    def test_import_without_env(self):
        """Test importing flexisettings without setting environment
        variable FLEXI_WRAPPED_MODULE
        """
        try:
            import flexisettings.settings
        except ImportError, e:
            if "Flexisettings cannot be imported, because env" in str(e):
                pass
            else:
                raise
        except:
            raise
        else:
            self.fail(
                "No 'ImportError' exception when FLEXI_WRAPPED_MODULE not set"
            )

    def test_import(self):
        """Test importing flexisettings with environment variable set"""
        os.environ.setdefault(self.envvar,
            "%s.settings" % self.test_project)
        try:
            import flexisettings.settings
        except:
            self.fail(sys.exc_info()[1])

    def test_import_local(self):
        """Check if local settings was imported"""
        import flexisettings.settings
        self.assertIn(
            '.'.join([self.test_project, 'settings_t']),
            flexisettings.settings._wrapped_modules
        )


# use the load_tests protocol to change the order of detected tests
def load_tests(loader, standard_tests, pattern):
    tests = unittest.TestSuite(map(ImportTestCase, ImportTestCase.tests))
    return tests

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import os
import sys

ENVIRONMENT_VARIABLE = "FLEXI_WRAPPED_MODULE"


class FlexiSettingsProxy(object):
    """Wrap configuration files following the app naming convention.

    For this app to work, the environment vairable
    'FLEXI_WRAPPED_MODULE' has to be set and point to django's settings.
    """

    _globals = {
        'FLEXI_SYS_PATH': ['apps', 'lib'],
        'FLEXI_AUTORELOAD': True,
        'FLEXI_LAYOUT_DISCOVERY': False,
        'FLEXI_MEDIA_FOLDER': 'media',
        'FLEXI_STATIC_FOLDER': 'static',
        'FLEXI_TEMPLATE_FOLDERS': ('templates', ),
    }
    _settings_path = None
    _wrapped_modules = {}

    def __init__(self):
        try:
            settings_module = os.environ[ENVIRONMENT_VARIABLE]
            if not settings_module:
                raise KeyError
        except KeyError:
            raise ImportError("Flexisettings cannot be imported, " \
                "because environment variable %s is undefined." \
                % ENVIRONMENT_VARIABLE
            )
        self._settings_module = settings_module
        self._settings_path = self._get_mod_dir(settings_module)

        self._import_settings()
        if self._globals['FLEXI_LAYOUT_DISCOVERY']:
            self._layout_discovery()
        if self._globals['FLEXI_AUTORELOAD']:
            for module, modfile in self._wrapped_modules.iteritems():
                # add this module to sys.modules
                sys.modules['flexisettings.wrapped.%s' % module] = MockModule(module, modfile)

    # old-style class attribute lookup
    def __getattr__(self, name):
        return self.__getattribute__(name)

    # new-style class attribute lookup
    def __getattribute__(self, name):
        # break recursion when calling private attributes
        if name.startswith('_') and name != name.upper():
            return object.__getattribute__(self, name)
        if name in self._globals:
            return self._globals[name]

        return object.__getattribute__(self, name)

    def __dir__(self):
        return dir(object) + self._globals.keys()

    def _get_package(self, module):
        """return a package string extracted from a module import string.
        This should work because the settings module should be on the
        python import path, this is not using the django mechanism of
        :code:`app.module`.
        See https://docs.djangoproject.com/en/1.5/topics/settings/#envvar-DJANGO_SETTINGS_MODULE

        :param module: a module string as used in import command
        :type module: str
        """
        return '.'.join(module.split('.')[:-1])

    def _get_mod_dir(self, module):
        """return the module folder

        :param module: a module string as used in import command
        :type module: str
        """
        package = self._get_package(module)
        mod = __import__(package)
        modfile = mod.__file__
        #FIXME: this won't work on a non *NiX system
        if not modfile.startswith('/'):
            modfile = os.path.abspath(modfile)
        return os.path.dirname(modfile)

    def _import_settings(self, quiet=True):
        """Get the settings imported in the proper order and depending
        on FLEXI_RUN_ENV variable.

        :param quiet: quietly fail if settings file does not exist
        :type quiet: bool
        """
        # import running environment
        self._import('env')
        # env variable supersedes file configuration
        if 'FLEXI_RUN_ENV' in os.environ:
            self._globals['FLEXI_RUN_ENV'] = os.environ['FLEXI_RUN_ENV']
        # import security file
        self._import('security', quiet)
        # import running env security file
        if 'FLEXI_RUN_ENV' in self._globals:
            self._import('security_%s' % self._globals['FLEXI_RUN_ENV'], quiet)
        # import main settings
        self._import('settings', False)
        # import running env settings
        if 'FLEXI_RUN_ENV' in self._globals:
            self._import('settings_%s' % self._globals['FLEXI_RUN_ENV'], quiet)

    def _import(self, modname, quiet=True):
        """import settings module and push configuration values to global scope

        The import order is as follow:
            * env.py
            * security.py
            * security_<FLEXI_RUN_ENV>.py, if FLEXI_RUN_ENV is set
            * settings.py, raises an IOError exception if missing
            * settings_<FLEXI_RUN_ENV>.py, if FLEXI_RUN_ENV is set

        :param modname: settings module name without package path, only
        the last part of a dotted name
        :type modname: str
        :param quiet: quietly fail if settings file does not exist
        :type quiet: bool
        """
        modfile = os.path.join(self._settings_path, modname) + '.py'
        globals_dict = dict(globals().items() + self._globals.items())
        # if the file doesn't exist but we want a quiet error
        if not os.path.isfile(modfile) and quiet:
            return

        execfile(modfile, globals_dict)
        # get which variables were set in modfile
        settings = set(globals_dict.keys()) - set(globals().keys())
        # push settings in _globals dict
        for setting in settings:
            if setting == setting.upper():
                self._globals[setting] = globals_dict[setting]
        # list this module as wrapped
        module = '.'.join([
            self._get_package(self._settings_module),
            modname
        ])
        self._wrapped_modules[module] = modfile

    def _site_dir(self, folder):
        """return an absolute path for a folder inside FLEXI_SITE_ROOT.

        :param folder: a folder name
        :type folder: str
        """
        return os.path.join(self._globals['FLEXI_SITE_ROOT'], folder)

    def _is_site_dir(self, folder):
        """returns a True if folder exists inside FLEXI_SITE_ROOT
        folder.

        :param folder: a folder name
        :type folder: str
        """
        return os.path.isdir(self._site_dir(folder))

    def _project_dir(self, folder):
        """return an absolute path for a folder inside
        FLEXI_PROJECT_ROOT.

        :param folder: a folder name
        :type folder: str
        """
        return os.path.join(self._globals['FLEXI_PROJECT_ROOT'], folder)

    def _is_project_dir(self, folder):
        """returns a True if folder exists inside FLEXI_PROJECT_ROOT
        folder.

        :param folder: a folder name
        :type folder: str
        """
        return os.path.isdir(self._project_dir(folder))

    def _layout_discovery(self):
        """perform layout detection:
            * set FLEXI_PROJECT_ROOT
            * set FLEXI_SITE_ROOT
            * add folders from FLEXI_SYS_PATH to sys.path
            * set MEDIA_ROOT
            * set STATIC_ROOT
            * add template folders to TEMPLATE_DIRS
        """
        # if project path is not specified, assume it's under the
        # settings folder
        if 'FLEXI_PROJECT_ROOT' not in self._globals:
            self._globals['FLEXI_PROJECT_ROOT'] = os.path.dirname(
                self._settings_path)
        self._project_path = self._globals['FLEXI_PROJECT_ROOT']
        # if site path is not specified, assume it is the same as
        # project path
        if 'FLEXI_SITE_ROOT' not in self._globals:
            self._globals['FLEXI_SITE_ROOT'] = self._project_path
        self._site_path = self._globals['FLEXI_SITE_ROOT']

        # add paths FLEXI_SYS_PATH to sys.path if they exist
        for folder in self._globals['FLEXI_SYS_PATH']:
            if self._is_project_dir(folder) \
                and self._project_dir(folder) not in sys.path:
                sys.path.insert(0, self._project_dir(folder))
            elif self._is_site_dir(folder) \
                and self._site_dir(folder) not in sys.path:
                sys.path.insert(0, self._site_dir(folder))

        # add media folder if MEDIA_ROOT is not already set or is empty
        if 'MEDIA_ROOT' not in self._globals \
            or not self._globals['MEDIA_ROOT']:
            folder = self._globals['FLEXI_MEDIA_FOLDER']
            if self._is_site_dir(folder):
                self._globals['MEDIA_ROOT'] = self._site_dir(folder)
            elif self._is_project_dir(folder):
                self._globals['MEDIA_ROOT'] = self._project_dir(folder)

        # add static folder if STATIC_ROOT is not already set or is empty
        if 'STATIC_ROOT' not in self._globals \
            or not self._globals['STATIC_ROOT']:
            folder = self._globals['FLEXI_STATIC_FOLDER']
            if self._is_site_dir(folder):
                self._globals['STATIC_ROOT'] = self._site_dir(folder)
            elif self._is_project_dir(folder):
                self._globals['STATIC_ROOT'] = self._project_dir(folder)

        # add templates folder if not in TEMPLATE_DIRS
        for folder in self._globals['FLEXI_TEMPLATE_FOLDERS']:
            if self._is_project_dir(folder) \
                not in self._globals['TEMPLATE_DIRS']:
                self._globals['TEMPLATE_DIRS'] += (
                    self._project_dir(folder),
                )


class MockModule(object):
    """An object mocking a module to use in sys.modules for django
    autoreload trickery.
    """
    def __init__(self, name, filename):
        self.__name__ = name
        self.__file__ = filename


# trick to replace the module by a class instance
# see http://mail.python.org/pipermail/python-ideas/2012-May/014969.html
# after importing flexisettings.settings, the module will actually be an
# instance of SettingsProxy.
sys.modules[__name__] = FlexiSettingsProxy()



# -*- coding: utf-8 -*-
import os
from distutils.core import setup, Command
import flexisettings

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# run unittest2 tests with 'python setup.py test'
class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from tests.run import runtests
        runtests()


setup(
    name=flexisettings.__title__,
    version=flexisettings.__version__,
    author=u'LeDaPei',
    author_email='code@ledapei.com',
    packages=['flexisettings'],
    url=u'https://github.com/ledapei/django-flexisettings',
    license='3-clause BSD licence, see LICENCE.txt',
    description='Django flexible settings with running environment support, separate security files and project layout detection.',
    long_description=README,
    cmdclass={ 'test': TestCommand },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
